package com.taauav.admin.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.entity.SysDict;
import com.taauav.admin.mapper.SysDictMapper;
import com.taauav.admin.service.ISysAdminService;
import com.taauav.admin.service.ISysDictService;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 系统字典表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2019-11-14
 */
@RestController
@RequestMapping("/dict")
public class SysDictController {
    @Autowired
    private ISysDictService dictService;
    @Autowired
    private Response response;
    @Autowired
    private SysDictMapper dictMapper;
    @Autowired
    private ISysAdminService adminService;

    private static final String controllerName = "dict";

    /**
     * 数据字典列表接口
     *
     * @param map
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) Map<String, String> map) {
        return dictService.getPageList(map);
    }

    /**
     * 编辑-提交接口
     *
     * @param map
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody SysDict map) {
        return dictService.editData(map);
    }

    /**
     * 编辑-获取原数据
     *
     * @param id
     * @return
     */
    @GetMapping("/getDictInfo")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response getDictInfo(Integer id) {
        if (id <= 0) {
            return response.failure("数据字典id不合法");
        }
        SysDict dict = dictService.getDictById(id);
        if (dict == null) {
            return response.failure("对应数据字典数据不存在");
        }
        return response.success("操作成功", dict);
    }

    /**
     * 新增数据字典接口
     *
     * @param map
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody SysDict map) {
        return dictService.editData(map);
    }

    /**
     * 删除数据字典接口
     *
     * @param id
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(@RequestBody Integer id) {
        if (id <= 0) {
            return response.failure("数据字典id不合法");
        }
        SysDict dict = dictService.getDictById(id);
        if (dict == null) {
            return response.failure("对应数据不存在");
        }
        dict.setMark(0);
        dict.setUpdateTime(DateUtil.now());
        dict.setUpdateUser(ShiroUtils.getAdminId());
        dictMapper.updateById(dict);
        return response.success("操作成功");
    }

    /**
     * 根据id获取子级数据
     *
     * @param id
     * @return
     */
    @GetMapping("/getChildList")
    public Response getChildByPid(Integer id) {
        if (id <= 0) {
            return response.failure("对应数据不存在");
        }
        SysDict info = dictMapper.selectById(id);
        if (info == null) {
            return response.failure("对应数据不存在");
        }
        if (info.getHasChild() == 1) {
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("mark", 1);
            wrapper.eq("pid", id);
            List<Map<String, Object>> list = dictMapper.selectMaps(wrapper);
            if (list != null) {
                for (Map<String, Object> val : list) {
                    Integer createUser = new Long(val.get("create_user").toString()).intValue();
                    val.put("createUserName", adminService.getRealNameById(createUser));
                    Integer createTime = new Long(val.get("create_time").toString()).intValue();
                    val.put("formatCreateTime", FunctionUtils.formatTime(createTime, "yyyy-MM-dd HH:mm:ss"));
                }
            }
            return response.success("操作成功", list);
        } else {
            return response.failure("无对应子级数据");
        }
    }

    @GetMapping("/pDictList")
    public Response getParentDictList() {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        wrapper.eq("has_child", 1);
        wrapper.eq("status", 1);
        wrapper.eq("pid", 0);
        List<SysDict> list = dictService.list(wrapper);
        List<Map<String, Object>> dictList = new ArrayList<>();
        if (list != null) {
            list.forEach(item -> {
                Map<String, Object> fMap = new HashMap<>();
                fMap.put("id", item.getId());
                fMap.put("name", item.getName());
                fMap.put("pid", item.getPid());
                QueryWrapper wrp = new QueryWrapper();
                wrp.eq("pid", item.getId());
                wrp.eq("mark", 1);
                wrp.eq("status", 1);
                wrp.eq("has_child", 1);
                List<SysDict> childList = dictService.list(wrp);
                List<Map<String, Object>> fChild = new ArrayList<>();
                if (childList != null) {
                    childList.forEach(val -> {
                        Map<String, Object> map = new HashMap<>();
                        map.put("id", val.getId());
                        map.put("name", val.getName());
                        map.put("pid", val.getPid());
                        QueryWrapper cond = new QueryWrapper();
                        cond.eq("pid", val.getId());
                        cond.eq("mark", 1);
                        cond.eq("status", 1);
                        cond.eq("has_child", 1);
                        List<Map<String, Object>> cList = dictMapper.selectList(cond);
                        map.put("children", cList);
                        fChild.add(map);
                    });
                }
                fMap.put("children", fChild);
                dictList.add(fMap);
            });
        }

        return response.success("操作成功", dictList);
    }
}
