package com.taauav.admin.controller;


import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.entity.TauvQuestionOptions;
import com.taauav.admin.service.ITauvQuestionOptionsService;
import com.taauav.common.util.FunctionUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 问题选项管理表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2019-11-13
 */
@RestController
@RequestMapping("/question")
public class TauvQuestionOptionsController {
    @Autowired
    private Response response;
    @Autowired
    private ITauvQuestionOptionsService iTauvQuestionOptionsService;

    private static final String controllerName = "question";

    /**
     * 问题选项数据列表
     * @param map
     * @return
     */
    @PostMapping("/options")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response options(@RequestBody(required = false) Map<String,String> map) {
        return iTauvQuestionOptionsService.getPageList(map);
    }

    /**
     * 编辑问题选项
     * @param questionOptions
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody @Valid TauvQuestionOptions questionOptions, BindingResult bindingResult) {
        if(bindingResult.hasErrors()){
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iTauvQuestionOptionsService.edit(questionOptions);
    }

    /**
     * 获取问题选项数据
     * @param id
     * @return
     */
    @GetMapping("/getQuestionOption")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response getQuestionOption(Integer id) {
        if (id <= 0) {
            return response.failure("对应数据不存在");
        }
        TauvQuestionOptions info = iTauvQuestionOptionsService.getInfoById(id);
        if (info == null) {
            return response.failure("对应数据不存在");
        }
        Map<String,Object> mapInfo = FunctionUtils.transBean2Map(info);
        mapInfo.put("categoryList",info.getCategoryList());
        mapInfo.put("typeList",info.getTypeList());
        return response.success(mapInfo);
    }


    /**
     * 添加数据
     * @param questionOptions
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody @Valid TauvQuestionOptions questionOptions, BindingResult bindingResult) {
        if(bindingResult.hasErrors()){
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iTauvQuestionOptionsService.add(questionOptions);
    }

    /**
     * 设置状态
     * @param map
     * @return
     */
    @PostMapping("/setStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(@RequestBody Map<String,String> map) {
        return iTauvQuestionOptionsService.setStatus(map);
    }

    /**
     * 基础数据
     * @return
     */
    @GetMapping("/baseData")
    public Response getBaseData() {
        return response.success(iTauvQuestionOptionsService.getBaseData());
    }
}
