package com.taauav.admin.service;

import com.taauav.common.bean.Response;
import com.taauav.admin.entity.LsAdmin;
import com.baomidou.mybatisplus.extension.service.IService;
import com.taauav.admin.query.LsAdminQuery;
import com.taauav.common.service.IBaseService;

import javax.validation.constraints.Max;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 外部人员表 服务类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
public interface ILsAdminService extends IBaseService<LsAdmin> {

    /**
     * 获取用户列表
     *
     * @param query 查询条件
     * @return
     */
    Response getList(LsAdminQuery query);

    /**
     * 添加用户
     *
     * @param entity 实体对象
     * @return
     */
    Response add(LsAdmin entity);

    /**
     * 获取数据详情
     *
     * @param id 用户ID
     * @return
     */
    Response info(Integer id);

    /**
     * 编辑用户
     *
     * @param entity 实体对象
     * @return
     */
    Response edit(LsAdmin entity);

    /**
     * 删除用户
     *
     * @param id 用户ID
     * @return
     */
    Response delete(Integer id);

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    Response setStatus(LsAdmin entity);

    /**
     * 重置密码
     *
     * @param entity 实体对象
     * @return
     */
    Response resetPwd(LsAdmin entity);

    /**
     * 基础数据
     *
     * @return
     */
    Map<String, Object> getBaseData();

    /**
     * 获取用户登录权限
     *
     * @param userId 用户ID
     * @return
     */
    List<Integer> getAdminAuthData(Integer userId);

    /**
     * 获取河长列表
     *
     * @return
     */
    Response getDriverManagerList();

}
