package com.taauav.admin.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.taauav.admin.service.IMapService;
import com.taauav.common.util.FunctionUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>高德地图</p>
 *
 * @author : dyg
 * @date : 2019-12-17 13:53
 **/
@Service("gaodeMap")
public class GaodeMapServiceImpl implements IMapService {
    private String key = "db4623de07899c6a7cf18eee6a1f1724";

    /**
     * 根据GPS坐标获取高德地图经纬度 高德（多个坐标对之间用”|”进行分隔最多支持40对坐标）
     * @param gpsInfo
     * @return
     */
    @Override
    public List<Map<String,String>> getGpsList(String gpsInfo) {
        List<Map<String,String>> list = new ArrayList<>();
        Map<String,String> map = new HashMap<>(2);
        String gaodeGps = geoconv(gpsInfo);

        JSONObject jsonObject = JSON.parseObject(gaodeGps);

        String status = jsonObject.getString("status");
        if (status.equalsIgnoreCase("1")) {
            String locations = jsonObject.getString("locations");
            String[] locationArr = locations.split(",");
            map.put("lon", locationArr[0]);
            map.put("lat", locationArr[1]);
            list.add(map);
        }
        return list;
    }
    /**
     * 根据坐标获取位置信息
     * @param longitude
     * @param latitude
     * @return
     */
    @Override
    public String getMapInfo(String longitude,String latitude)
    {
        String address = null;
        try {
            longitude = FunctionUtils.round(new Double(longitude),6);
            latitude = FunctionUtils.round(new Double(latitude),6);
            String query = longitude+","+latitude;
            String locJson = FunctionUtils.getUrl("https://restapi.amap.com/v3/geocode/regeo","key="+key+"&location="+query);
            JSONObject jsonObject =  JSON.parseObject(locJson);
            String status = jsonObject.getString("status");
            if ("1".equalsIgnoreCase(status)) {
                JSONObject regeoObject = jsonObject.getJSONObject("regeocode");
                address = regeoObject.getString("formatted_address");
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return address;
    }

    /**
     * GPS坐标转成高德坐标
     * @param gpsStr
     * @return
     * @throws Exception
     */
    @Override
    public String geoconv(String gpsStr){
        String locJson = null;
        try {
            locJson = FunctionUtils.getUrl("https://restapi.amap.com/v3/assistant/coordinate/convert","locations="+gpsStr+"&coordsys=gps&key="+key);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return locJson;
    }
}
