package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.constant.CommonConstant;
import com.taauav.admin.entity.LsAuthGroup;
import com.taauav.admin.mapper.LsAuthGroupMapper;
import com.taauav.admin.service.ILsAuthGroupService;
import com.taauav.admin.service.ISysCityService;
import com.taauav.admin.vo.LsAdminListVo;
import com.taauav.admin.vo.LsDriverManagerListVo;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.StringUtils;
import com.taauav.admin.entity.LsAdmin;
import com.taauav.admin.mapper.LsAdminMapper;
import com.taauav.admin.query.LsAdminQuery;
import com.taauav.admin.service.ILsAdminService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 外部人员表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@Service
public class LsAdminServiceImpl extends BaseServiceImpl<LsAdminMapper, LsAdmin> implements ILsAdminService {

    @Autowired
    private LsAdminMapper lsAdminMapper;

    @Autowired
    private Response response;

    @Autowired
    private ISysCityService cityService;

    @Autowired
    private ILsAuthGroupService authGroupService;

    @Autowired
    private LsAuthGroupMapper authGroupMapper;
    private QueryWrapper<LsAdmin> queryWrapper;

    /**
     * 获取数据列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(LsAdminQuery query) {
        // 查询条件
        QueryWrapper<LsAdmin> queryWrapper = new QueryWrapper<>();
        // 用户名称
        if (!StringUtils.isEmpty(query.getRealname())) {
            queryWrapper.like("realname", query.getRealname());
        }
        // 区划ID
        if (query.getDriverArea() != null) {
            queryWrapper.eq("driver_area", query.getDriverArea());
        }
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询数据
        IPage<LsAdmin> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<LsAdmin> data = lsAdminMapper.selectPage(page, queryWrapper);
        List<LsAdmin> lsAdminList = data.getRecords();
        List<LsAdminListVo> lsAdminListVoList = new ArrayList<>();
        if (!lsAdminList.isEmpty()) {
            lsAdminList.forEach(item -> {
                LsAdminListVo lsAdminListVo = new LsAdminListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, lsAdminListVo);
                // 区划描述
                String cityName = cityService.getCityName(item.getDriverArea());
                lsAdminListVo.setDriverAreaName(cityName);
                // 角色描述
                String authGroupName = authGroupService.getAuthGroupName(item.getAuthGroup());
                lsAdminListVo.setAuthGroupName(authGroupName);
                lsAdminListVoList.add(lsAdminListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", lsAdminListVoList);
        return response.success(result);
    }

    /**
     * 添加用户
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(LsAdmin entity) {
        if (StringUtils.isEmpty(entity.getPassword())) {
            // 初始密码:123456
            entity.setPassword(FunctionUtils.password("123456"));
        } else {
            entity.setPassword(FunctionUtils.password(entity.getPassword()));
        }
        boolean result = this.addData(entity);
        if (!result) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取用户详情
     *
     * @param id 用户ID
     * @return
     */
    @Override
    public Response info(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("用户ID不存在");
        }
        LsAdmin entity = lsAdminMapper.selectById(id);
        if (entity == null) {
            return response.failure("用户信息不存在");
        }
        return response.success(entity);
    }

    /**
     * 编辑用户
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(LsAdmin entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("用户ID不存在");
        }
        // 设置密码并加密
        if (!StringUtils.isEmpty(entity.getPassword())) {
            entity.setPassword(FunctionUtils.password(entity.getPassword()));
        }
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除用户
     *
     * @param id 用户ID
     * @return
     */
    @Override
    public Response delete(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("用户ID不存在");
        }
        LsAdmin entity = lsAdminMapper.selectById(id);
        if (entity == null) {
            return response.failure("用户信息不存在");
        }
        boolean result = this.deleteById(id, entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(LsAdmin entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("用户ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("用户状态不能为空");
        }
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

    /**
     * 重置密码
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response resetPwd(LsAdmin entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("用户ID不存在");
        }
        // 重置密码：123456
        entity.setPassword(FunctionUtils.password("123456"));
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("重置失败");
        }
        return response.success("重置成功");
    }

    /**
     * 获取基础数据
     *
     * @return
     */
    @Override
    public Map<String, Object> getBaseData() {
        Map<String, Object> map = new HashMap<>(2);
        map.put("level_list", CommonConstant.LSADMIN_LEVEL_LIST);
        map.put("area_list", cityService.getCityList(BigInteger.valueOf(Long.valueOf("320117"))));
        return map;
    }

    /**
     * 获取人员数据权限
     *
     * @param userId 用户ID
     * @return
     */
    @Override
    public List<Integer> getAdminAuthData(Integer userId) {
        LsAdmin adminInfo = lsAdminMapper.selectById(userId);
        if (adminInfo == null) {
            return null;
        }
        if (!StringUtils.isEmpty(adminInfo.getAuthGroup())) {
            return null;
        }
        String[] authGroup = adminInfo.getAuthGroup().split(",");
        ArrayList arrayList = new ArrayList();
        for (String s : authGroup) {
            arrayList.add(s);
        }
        // 查询条件
        QueryWrapper<LsAuthGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", arrayList);
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);

        // 查询数据
        List<Integer> authDataList = new ArrayList<>();
        List<LsAuthGroup> authGroupList = authGroupMapper.selectList(queryWrapper);
        if (!authGroupList.isEmpty()) {
            for (LsAuthGroup lsAuthGroup : authGroupList) {
                if (lsAuthGroup.getAuthData() == null || lsAuthGroup.getAuthData() <= 0) {
                    continue;
                }
                authDataList.add(Integer.valueOf(lsAuthGroup.getAuthData()));
            }
        }
        return authDataList;
    }

    /**
     * 获取河长列表
     *
     * @return
     */
    @Override
    public Response getDriverManagerList() {
        // 查询条件
        QueryWrapper<LsAdmin> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");
        queryWrapper.select("id", "realname");

        // 查询数据
        List<LsAdmin> lsAdminList = lsAdminMapper.selectList(queryWrapper);
        List<LsDriverManagerListVo> driverManagerListVoList = new ArrayList<>();
        if (!lsAdminList.isEmpty()) {
            lsAdminList.forEach(item -> {
                LsDriverManagerListVo driverManagerListVo = new LsDriverManagerListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, driverManagerListVo);
                driverManagerListVoList.add(driverManagerListVo);

            });
        }
        return response.success(driverManagerListVoList);
    }
}
