package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.SysCity;
import com.taauav.admin.mapper.SysAdminMapper;
import com.taauav.admin.mapper.SysCityMapper;
import com.taauav.admin.mapper.TauvDriverMapper;
import com.taauav.admin.service.ISysCityService;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;

/**
 * <p>
 * 城市表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-13
 */
@Service
public class SysCityServiceImpl extends ServiceImpl<SysCityMapper, SysCity> implements ISysCityService {
    @Autowired
    private SysCityMapper sysCityMapper;
    @Autowired
    private SysAdminMapper adminMapper;
    @Autowired
    private TauvDriverMapper tauvDriverMapper;
    @Resource
    private Response response;

    /**
     * 根据上级编号获取数据
     *
     * @param pid
     * @return
     */
    @Override
    public List<Map<String, Object>> getCityListByPid(Integer pid) {
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        queryWrapper.select("id,pid,name,longitude,latitude");
        List<Map<String, Object>> list = sysCityMapper.selectMaps(queryWrapper);
        return list;
    }

    /**
     * 获取城市列表
     *
     * @param pid 上级ID
     * @return
     */
    @Override
    public List<SysCity> getCityList(BigInteger pid) {
        List<SysCity> cityList = new ArrayList<>();
        if (pid.toString().equals("320117")) {
            QueryWrapper<SysCity> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.eq("id", pid);
            queryWrapper2.eq("mark", 1);
            queryWrapper2.select("id,pid,name,longitude,latitude");
            SysCity city = sysCityMapper.selectOne(queryWrapper2);
            cityList.add(city);
        }
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        queryWrapper.select("id,pid,name,longitude,latitude");
        List<SysCity> list = sysCityMapper.selectList(queryWrapper);
        if (!list.isEmpty()) {
            // 获取子级
            list.forEach(item -> {
                cityList.add(item);
                List<SysCity> children = this.getCityList(BigInteger.valueOf(Long.valueOf(item.getId().toString())));
                if (!children.isEmpty()) {
                    for (SysCity child : children) {
                        cityList.add(child);
                    }
                }
            });
        }
        return cityList;
    }

    /**
     * 根据父级id和用户城市权限获取城市列表
     * 备注：待完善
     *
     * @param pid
     * @return
     */
    @Override
    public List<Map<String, Object>> getCityListByPidAndAdmin(Integer pid) {
        Integer adminId = ShiroUtils.getAdminId();
        SysAdmin admin = adminMapper.selectById(adminId);
        if (!"".equals(admin.getCityIds())) {
            QueryWrapper<SysCity> wrapper = new QueryWrapper<>();
            if (!StringUtils.isEmpty(admin.getCityIds())) {
                String[] ids = admin.getCityIds().split(",");
                wrapper.in("id", ids);
            }
            wrapper.eq("pid", pid);
            wrapper.eq("mark", 1);
            wrapper.select("id,pid,name,longitude,latitude");
            List<Map<String, Object>> list = sysCityMapper.selectMaps(wrapper);
            return list;
        } else {
            return null;
        }
    }

    @Override
    public SysCity getInfoById(BigInteger id) {
        return sysCityMapper.selectById(id);
    }

    /**
     * 获取城市各辖区河道信息
     *
     * @param cityId
     * @return
     */
    @Override
    public Response getCityDriverList(Integer cityId) {
        List<Map<String, Object>> cityList = getCityListByPid(cityId);
        if (cityList != null) {
            for (Map<String, Object> city : cityList) {
                QueryWrapper wrapper = new QueryWrapper();
                wrapper.eq("mark", 1);
                wrapper.eq("status", 1);
                wrapper.eq("driver_area", city.get("id"));
                Integer num = tauvDriverMapper.selectCount(wrapper);
                city.put("count", num);
            }
        }
        return response.success(cityList);
    }

    /**
     * 根据城市ID获取城市名称
     *
     * @param cityId 城市ID
     * @return
     */
    @Override
    public String getCityName(BigInteger cityId) {
        List<String> nameList = new ArrayList<>();
        while (cityId != null && cityId.compareTo(BigInteger.valueOf(320117)) >= 0) {
            SysCity cateInfo = sysCityMapper.selectById(cityId);
            if (cateInfo != null) {
                nameList.add(cateInfo.getName());
                cityId = cateInfo.getPid();
            } else {
                cityId = null;
            }
        }
        // 使用集合工具实现数组翻转
        Collections.reverse(nameList);
        return org.apache.commons.lang3.StringUtils.join(nameList.toArray(), ">>");
    }
}
