package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.SysLevelConstant;
import com.taauav.common.constant.SysPositionConstant;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.SysPosition;
import com.taauav.admin.mapper.SysAdminMapper;
import com.taauav.admin.mapper.SysPositionMapper;
import com.taauav.admin.service.ISysPositionService;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import javax.validation.Validator;
import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 职位表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-12
 */
@Service
public class SysPositionServiceImpl extends BaseServiceImpl<SysPositionMapper, SysPosition> implements ISysPositionService {
    @Autowired
    private SysPositionMapper positionMapper;
    @Autowired
    private SysAdminMapper adminMapper;
    @Resource
    private Response response;
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Resource
    private Validator validator;

    /**
     * 获取列表数据
     *
     * @param map
     * @return
     */
    @Override
    public Response index(Map<String, String> map) {
        Integer pageNo = 1;
        if (map == null) {
            pageNo = 1;
        } else {
            pageNo = map.containsKey("page") && Integer.valueOf(map.get("page")) > 0 ? Integer.valueOf(map.get("page")) : 1;
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        if (map != null) {
            //关键词筛选
            String keywords = "keywords";
            if (map.containsKey(keywords)) {
                if (FunctionUtils.isNumeric(map.get(keywords))) {
                    wrapper.eq("id", Integer.valueOf(map.get(keywords)));
                } else {
                    wrapper.like("name", map.get(keywords));
                }
            }
            //状态筛选
            String status = "status";
            if (map.containsKey(status) && FunctionUtils.isNumeric(map.get(status))) {
                wrapper.eq("status", Integer.valueOf(map.get(status)));
            }
            //添加人筛选
            String createUser = "createUser";
            if (map.containsKey(createUser) && FunctionUtils.isNumeric(map.get(createUser))) {
                wrapper.eq("create_user", Integer.valueOf(map.get(createUser)));
            }
            //添加时间筛选
            String startTime = "startTime";
            String endTime = "endTime";
            if (map.containsKey(startTime) && map.containsKey(endTime)) {
                if (FunctionUtils.isNumeric(map.get(startTime)) && FunctionUtils.isNumeric(map.get(endTime))) {
                    wrapper.between("create_time", Integer.valueOf(map.get(startTime)), Integer.valueOf(map.get(endTime)));
                }
            }
            //每页数量
            String pgSize = "pageSize";
            if (map.containsKey(pgSize) && Integer.valueOf(map.get(pgSize)) > 0) {
                pageSize = Integer.valueOf(map.get(pgSize));
            }
        }
        //排序
        wrapper.orderByAsc("sort");
        //判断页数
        double total = positionMapper.selectCount(wrapper);
        Integer totalPage = (int) Math.ceil(total / pageSize);
        if (pageNo > totalPage) {
            pageNo = totalPage;
        }
        IPage<SysPosition> page = new Page<>(pageNo, pageSize);
        IPage data = this.page(page, wrapper);

        Map<String, Object> list = new HashMap<>(6);
        list.put("total", data.getTotal());
        list.put("size", data.getSize());
        list.put("current", data.getCurrent());
        list.put("pages", data.getPages());
        if (data.getRecords() != null) {
            List<SysPosition> positionList = data.getRecords();
            List<Object> records = new ArrayList<>();
            positionList.forEach(val -> {
                Map<String, Object> mp = new HashMap<>(11);
                mp.put("id", val.getId());
                mp.put("name", val.getName());
                mp.put("remark", val.getRemark());
                mp.put("level", val.getLevel());
                mp.put("sort", val.getSort());
                mp.put("status", val.getStatus());
                mp.put("statusText", FunctionUtils.getArrayText(String.valueOf(val.getStatus()), SysPositionConstant.getPositionStatus()));
                mp.put("createUser", val.getCreateUser());
                SysAdmin admin = adminMapper.selectById(val.getCreateUser());
                if (admin != null) {
                    mp.put("createUsername", admin.getRealname());
                } else {
                    mp.put("createUsername", "");
                }
                mp.put("createTime", val.getCreateTime());
                mp.put("formatCreateTime", val.getCreateTime());
                records.add(mp);
            });
            list.put("records", records);
        }
        return response.success("操作成功", list);
    }

    /**
     * 添加、编辑
     *
     * @param map
     * @return
     */
    @Override
    public Response edit(SysPosition map) {
        //必填字段及格式校验
        Set<ConstraintViolation<SysPosition>> violations = validator.validate(map);
        for (ConstraintViolation<SysPosition> violation : violations) {
            if (!"".equals(violation.getMessage())) {
                return response.failure(violation.getMessage());
            }
        }
        SysPosition position = new SysPosition();
        position.setName(map.getName());
        String remark = map.getRemark();
        if (remark != null) {
            position.setRemark(remark);
        }
        position.setStatus(map.getStatus());
        position.setLevel(map.getLevel());
        position.setSort(map.getSort());
        Integer id = map.getId();
        Boolean res;
        if (id != null && id > 0) {
            position.setUpdateTime(DateUtil.now());
            position.setUpdateUser(ShiroUtils.getAdminId());
            position.setId(id);
            res = updateById(position);
        } else {
            position.setCreateTime(DateUtil.now());
            position.setCreateUser(ShiroUtils.getAdminId());
            res = save(position);
        }
        if (res.equals(false)) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }

    /**
     * 根据id获取对应职级数据
     *
     * @param id
     * @return
     */
    @Override
    public Response getPositionInfo(Integer id) {
        if (id <= 0) {
            return response.failure("职级id必须大于0");
        }
        SysPosition position = positionMapper.selectById(id);
        if (position == null) {
            return response.failure("对应职级数据不存在");
        }
        Map<String, Object> info = new HashMap<>(14);
        info.put("id", position.getId());
        info.put("name", position.getName());
        info.put("status", position.getStatus());
        info.put("statusText", FunctionUtils.getArrayText(String.valueOf(position.getStatus()), SysLevelConstant.getLevelStatus()));
        info.put("remark", position.getRemark());
        info.put("level", position.getLevel());
        info.put("sort", position.getSort());
        info.put("createUser", position.getCreateUser());
        String createUserName = "";
        if (position.getCreateUser() > 0) {
            SysAdmin admin = adminMapper.selectById(position.getCreateUser());
            if (admin != null) {
                createUserName = admin.getRealname();
            }
        }
        info.put("createUserName", createUserName);
        info.put("createTime", position.getCreateTime());
        String formatCreateTime = "";
        if (position.getCreateTime() != null) {
            formatCreateTime = position.getCreateTime().toString();
        }
        info.put("formatCreateTime", formatCreateTime);
        info.put("updateUser", position.getUpdateUser());
        String updateUserName = "";
        if (position.getUpdateUser() > 0) {
            SysAdmin admin = adminMapper.selectById(position.getUpdateUser());
            if (admin != null) {
                updateUserName = admin.getRealname();
            }
        }
        info.put("updateUserName", updateUserName);
        info.put("updateTime", position.getUpdateTime());
        String formatUpdateTime = "";
        if (position.getUpdateTime() != null) {
            formatUpdateTime = position.getUpdateTime().toString();
        }
        info.put("formatUpdateTime", formatUpdateTime);
        return response.success("操作成功", info);
    }

    /**
     * 根据id删除职级
     *
     * @param id
     * @return
     */
    @Override
    public Response drop(Integer id) {
        if (id <= 0) {
            return response.failure("岗位id必须大于0");
        }
        SysPosition position = positionMapper.selectById(id);
        if (position == null) {
            return response.failure("岗位数据不存在");
        }
        position.setMark(0);
        position.setUpdateUser(ShiroUtils.getAdminId());
        position.setUpdateTime(DateUtil.now());
        Boolean res = updateById(position);
        if (res.equals(false)) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }

    /**
     * 修改职级状态
     *
     * @param id
     * @param status
     * @return
     */
    @Override
    public Response changeStatus(Integer id, Integer status) {
        if (id < 1) {
            return response.failure("岗位id必须大于0");
        }
        SysPosition info = positionMapper.selectById(id);
        if (info == null) {
            return response.failure("岗位数据不存在");
        }
        Integer st = 0;
        if (status == 1) {
            st = 2;
        } else {
            st = 1;
        }
        info.setStatus(st);
        info.setUpdateUser(ShiroUtils.getAdminId());
        info.setUpdateTime(DateUtil.now());
        Boolean res = updateById(info);
        if (res.equals(false)) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }
}
