package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.taauav.admin.entity.SysCity;
import com.taauav.admin.mapper.SysCityMapper;
import com.taauav.admin.service.ITauvCityService;
import com.taauav.admin.vo.TauvCityListVo;
import com.taauav.common.bean.Response;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 用户组表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@Service
public class TauvCityServiceImpl extends ServiceImpl<SysCityMapper, SysCity> implements ITauvCityService {

    @Autowired
    private SysCityMapper sysCityMapper;

    @Autowired
    private Response response;

    /**
     * 获取机构列表
     *
     * @return
     */
    @Override
    public Response getList() {
        // 查询条件
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", BigInteger.valueOf(320117));
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询条件
        List<SysCity> cityList = sysCityMapper.selectList(queryWrapper);
        return response.success(cityList);
    }

    /**
     * 获取子级城市列表
     *
     * @param pid 上级ID
     * @return
     */
    @Override
    public Response getChildCityList(BigInteger pid) {
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");
        List<SysCity> agencyList = sysCityMapper.selectList(queryWrapper);
        return response.success(agencyList);
    }

    /**
     * 添加机构
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(SysCity entity) {
        if (entity == null) {
            return response.failure("实体对象不能为空");
        }
        if (entity.getPid() == null) {
            entity.setLevel(1);
        } else {
            SysCity agency = sysCityMapper.selectById(entity.getPid());
            if (entity != null) {
                entity.setLevel(agency.getLevel() + 1);
            }
        }
        entity.setCreateTime(DateUtil.now());
        entity.setCreateUser(ShiroUtils.getAdminId());
        int result = sysCityMapper.insert(entity);
        if (result == 0) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取机构详情
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response info(BigInteger id) {
        if (id == null) {
            return response.failure("机构ID不存在");
        }
        SysCity entity = this.getById(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        return response.success(entity);
    }

    /**
     * 编辑机构
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(SysCity entity) {
        if (entity == null || entity.getId() == null) {
            return response.failure("实体对象不能为空");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(DateUtil.now());
        boolean result = this.updateById(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除机构
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response delete(BigInteger id) {
        if (id == null) {
            return response.failure("机构ID不存在");
        }
        SysCity entity = sysCityMapper.selectById(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(DateUtil.now());
        entity.setMark(0);
        boolean result = this.updateById(entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(SysCity entity) {
        if (entity.getId() == null) {
            return response.failure("设备ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("状态不存在");
        }
        int result = sysCityMapper.updateById(entity);
        if (result == 0) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

}
