package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.TauvReportEvaluation;
import com.taauav.admin.mapper.TauvReportEvaluationMapper;
import com.taauav.admin.service.ISysAdminService;
import com.taauav.admin.service.ITauvReportEvaluationService;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 报告描述(评价)内容管理表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-14
 */
@Service
public class TauvReportEvaluationServiceImpl extends BaseServiceImpl<TauvReportEvaluationMapper, TauvReportEvaluation> implements ITauvReportEvaluationService {
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Autowired
    private Response response;
    @Autowired
    private TauvReportEvaluationMapper reportEvaluationMapper;
    @Autowired
    private ISysAdminService iSysAdminService;

    /**
     * 分页数据
     *
     * @param map
     * @return
     */
    @Override
    public Response getPageList(Map<String, String> map) {
        QueryWrapper<TauvReportEvaluation> wrapper = new QueryWrapper<>();
        wrapper.eq("mark", 1);
        Integer pageNo = 1;
        if (map != null) {
            //每页数量
            String size = "pageSize";
            if (map.containsKey(size) && Integer.valueOf(map.get(size)) > 0) {
                pageSize = Integer.valueOf(map.get(size));
            }
            //页数
            String page = "page";
            if (map.containsKey(page) && !"".equals(map.get(page)) && Integer.valueOf(map.get(page)) > 0) {
                pageNo = Integer.valueOf(map.get("page"));
            }
        }
        IPage<TauvReportEvaluation> page = new Page<>(pageNo, pageSize);
        IPage<Map<String, Object>> data = reportEvaluationMapper.selectMapsPage(page, wrapper);
        List<Map<String, Object>> dataList = data.getRecords();
        if (!StringUtils.isEmpty(dataList)) {
            TauvReportEvaluation reportEvaluation = new TauvReportEvaluation();
            Map<Integer, String> statusList = reportEvaluation.getStatusList();
            for (Map<String, Object> dt : dataList) {
                dt.put("status_text", StringUtils.isEmpty(statusList.get(dt.get("status"))) ? "" : statusList.get(dt.get("status")));
                if (StringUtils.isEmpty(dt.get("create_time"))) {
                    dt.put("format_create_time", "");
                } else {
                    String createTime = dt.get("create_time").toString();
                    dt.put("format_create_time", FunctionUtils.formatTime(Integer.valueOf(createTime), "yyyy-MM-dd HH:mm:ss"));
                }
                dt.put("format_create_user", "");
                if (!StringUtils.isEmpty(dt.get("create_user"))) {
                    Integer createUser = Integer.valueOf(dt.get("create_user").toString());
                    SysAdmin adminInfo = iSysAdminService.getById(createUser);
                    if (!StringUtils.isEmpty(adminInfo)) {
                        dt.put("format_create_user", adminInfo.getRealname());
                    }
                }
            }
        }
        data.setRecords(dataList);
        return response.success(data);
    }

    /**
     * 添加数据
     *
     * @param reportEvaluation
     * @return
     */
    @Override
    public Response add(TauvReportEvaluation reportEvaluation) {
        addData(reportEvaluation);
        return response.success("添加成功");
    }

    /**
     * 修改数据
     *
     * @param reportEvaluation
     * @return
     */
    @Override
    public Response edit(TauvReportEvaluation reportEvaluation) {
        editData(reportEvaluation);
        return response.success("编辑成功");
    }

    /**
     * 设置状态
     *
     * @param map
     * @return
     */
    @Override
    public Response setStatus(Map<String, String> map) {
        if (StringUtils.isEmpty(map)) {
            return response.failure("参数不能为空");
        }
        String idStr = map.getOrDefault("id", "");
        String statusStr = map.getOrDefault("status", "");
        if (StringUtils.isEmpty(idStr)) {
            return response.failure("编号不存在");
        }
        if (StringUtils.isEmpty(statusStr)) {
            return response.failure("状态不能为空");
        }
        if (!("1".equals(statusStr) || "2".equals(statusStr))) {
            return response.failure("状态值不合法");
        }
        Integer id = Integer.valueOf(idStr);
        if (StringUtils.isEmpty(getInfoById(id))) {
            return response.failure("数据不存在");
        }
        TauvReportEvaluation reportEvaluation = new TauvReportEvaluation();
        reportEvaluation.setId(Integer.valueOf(idStr));
        reportEvaluation.setStatus(Integer.valueOf(statusStr));
        editData(reportEvaluation);
        return response.success("操作成功");
    }

    /**
     * 根据编号获取数据
     *
     * @param id
     * @return
     */
    @Override
    public TauvReportEvaluation getInfoById(Integer id) {
        if (StringUtils.isEmpty(id)) {
            return null;
        }
        return reportEvaluationMapper.selectById(id);
    }

    /**
     * 根据状态获取数据列表
     *
     * @param status
     * @return
     */
    @Override
    public List<Map<String, Object>> getDataByStatus(Integer status) {
        QueryWrapper<TauvReportEvaluation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", status);
        queryWrapper.eq("mark", 1);
        queryWrapper.select("id,content");
        List<Map<String, Object>> list = reportEvaluationMapper.selectMaps(queryWrapper);
        return list;
    }
}
