package com.taauav.common.util;

import com.taauav.admin.entity.SysAdmin;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.Authenticator;
import org.apache.shiro.authc.LogoutAware;
import org.apache.shiro.session.Session;
import org.apache.shiro.subject.SimplePrincipalCollection;
import org.apache.shiro.subject.support.DefaultSubjectContext;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.crazycake.shiro.RedisSessionDAO;

import java.util.Collection;
import java.util.Objects;

/**
 * shiro工具类
 * @author dyg
 * @date 2019/11/06
 */
@Slf4j
public class ShiroUtils {

    /** 私有构造器 **/
    private ShiroUtils(){ }

    private static RedisSessionDAO redisSessionDAO = SpringUtil.getBean(RedisSessionDAO.class);

    /**
     * 获取当前用户信息
     * @return
     */
    public static SysAdmin getAdminInfo() {
        SysAdmin admin = (SysAdmin) SecurityUtils.getSubject().getPrincipal();
        return admin;
    }

    /**
     * 获取用户编号
     * @return
     */
    public static Integer getAdminId() {
        SysAdmin admin = getAdminInfo();
        return admin.getId();
    }

    /**
     * 删除用户缓存信息
     * @param username
     * @param isRemoveSession
     */
    public static void deleteCache(String username, boolean isRemoveSession) {
        Session session = null;
        Collection<Session> sessions = redisSessionDAO.getActiveSessions();
        SysAdmin sysAdmin;
        Object attribute = null;
        for (Session sessionInfo : sessions) {
            log.warn("attribute=" + DefaultSubjectContext.PRINCIPALS_SESSION_KEY);
            attribute = sessionInfo.getAttribute(DefaultSubjectContext.PRINCIPALS_SESSION_KEY);
            if (null == attribute) {
                continue;
            }
            sysAdmin = (SysAdmin)((SimplePrincipalCollection) attribute).getPrimaryPrincipal();
            if (null == sysAdmin) {
                continue;
            }
            if (Objects.equals(sysAdmin.getUsername(), username)) {
                session = sessionInfo;
                break;
            }
        }
        if (null == session || null == attribute) {
            return;
        }
        //删除session
        if (isRemoveSession) {
            redisSessionDAO.delete(session);
        }
        //删除cache，在访问受限接口时会重新授权
        DefaultWebSecurityManager securityManager = (DefaultWebSecurityManager) SecurityUtils.getSecurityManager();
        Authenticator authc = securityManager.getAuthenticator();
        ((LogoutAware) authc).onLogout((SimplePrincipalCollection) attribute);
    }

}
